//
//  HTViewHierarchyCtl.m
//  ViewHierarchy
//
//  Created by Cameron Hayne on 29/05/06.
//  Copyright 2006 Hayne of Tintagel.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



// *** INSTRUCTIONS ***
// To make use of this within your project, add the files
// HTViewHierarchyCtl.h, HTViewHierarchyCtl.m, and HTViewHierarchy.nib
// into your project, and then add the following line in an appropriate place
// (e.g. applicationDidFinishLaunching):
// [NSBundle loadNibNamed: @"HTViewHierarchy" owner: self];
// The "View Hierarchy" window will then show up automatically.

#import "HTViewHierarchyCtl.h"

@interface HTViewHierarchyCtl (Private)
- (NSArray *)childrenForOutlineViewItem:(id)item;
- (NSString *)valueForView:(id)view;
- (NSWindow *)windowWithWindowNum:(int)aWindowNum;
- (void)flashView:(NSView *)aView;
@end

@implementation HTViewHierarchyCtl

- (void)awakeFromNib
{
    //NSLog(@"HTViewHierarchyCtl.awakeFromNib");
    
    [[NSNotificationCenter defaultCenter]
	addObserver: self
	   selector: @selector(refreshWindowList:)
	       name: NSPopUpButtonWillPopUpNotification
	     object: windowPopup];
}

#pragma mark accessors

- (void)setCurrWindow:(NSWindow *)aWindow
{
    [aWindow retain];
    [currWindow release];
    currWindow = aWindow;
}

- (NSWindow *)currWindow
{
    return currWindow;
}

- (NSView *)currContentView
{
    return [[self currWindow] contentView];
}

#pragma mark outlineView datasource methods

// convenience method
- (NSArray *)childrenForOutlineViewItem:(id)item
{
    NSView *contentView = [self currContentView];
    if (contentView == nil)
    {
        return [NSArray array]; // empty array
    }
    
    NSArray *children;
    if (item == nil) // top level
    {
        children = [NSArray arrayWithObject: contentView];
    }
    else // item is an NSView
    {
        children = [item subviews];
    }
    
    return children;
}

- (int)outlineView:(NSOutlineView *)ov numberOfChildrenOfItem:(id)item
{
    int numChildren = [[self childrenForOutlineViewItem: item] count];
    return numChildren;
}

- (BOOL)outlineView:(NSOutlineView *)ov isItemExpandable:(id)item
{
    int numChildren = [[self childrenForOutlineViewItem: item] count];
    return (numChildren > 0 ? YES : NO);
}

- (id)outlineView:(NSOutlineView *)ov child:(int)index ofItem:(id)item
{
    NSView *view = [[self childrenForOutlineViewItem: item] objectAtIndex: index];
    //NSLog(@"child: %d OfItem: %@ is view: %@", index, item, view);
    return view;
}

- (id)outlineView:(NSOutlineView *)ov objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
    NSString *colId = [tableColumn identifier];
    if ([colId isEqual: @"class"])
    {
        NSString *className = [item className];
        NSString *value = [self valueForView: item];
        if (value != nil)
        {
            return [NSString stringWithFormat: @"%@ \"%@\"", className, value];
        }
        else
        {
            return className;
        }
    }
    else if ([colId isEqual: @"frame"])
    {
        return NSStringFromRect([item frame]);
    }
    else if ([colId isEqual: @"bounds"])
    {
        return NSStringFromRect([item bounds]);
    }
    else
    {
        return @"unknown";
    }
}

- (BOOL)outlineView:(NSOutlineView *)ov shouldSelectItem:(id)item
{
    [self flashView: item];
    return NO;
}

#pragma mark utility methods

// utility method
- (NSString *)valueForView:(id)view
{
    NSString *value = nil;
    if ([view respondsToSelector: @selector(title)])
    {
        value = [view title];
    }
    else if ([view respondsToSelector: @selector(stringValue)])
    {
        value = [view stringValue];
    }
    
    int maxLen = 20;
    if ([value length] > maxLen)
    {
        value = [value substringToIndex: maxLen];
    }
    
    //NSLog(@"valueForView: %@ len: %d", value, [value length]);
    return value;
}

// utility method
- (NSWindow *)windowWithWindowNum:(int)aWindowNum
{
    NSArray *windows = [NSApp windows];
    NSEnumerator *winEnum = [windows objectEnumerator];
    NSWindow *win;
    while (win = (NSWindow *)[winEnum nextObject])
    {
        if ([win windowNumber] == aWindowNum)
        {
            return win;
        }
    }
    return nil;
}

// utility method
- (void)flashView:(NSView *)aView
{
    if ([aView canDraw] == NO)
    {
        NSBeep();
        return;
    }
    
    [aView lockFocus];
    [[NSColor redColor] set];
    NSRectFill([aView bounds]);
    [aView unlockFocus];
    [[aView window] flushWindow];
    [aView setNeedsDisplay: YES];
}

#pragma mark update methods

- (void)refreshWindowList:(NSNotification *)notification
{
    //NSLog(@"windowPopup clicked on");
    NSArray *windows = [NSApp windows];
    NSEnumerator *winEnum = [windows objectEnumerator];
    NSWindow *win;
    
    int selected = [windowPopup selectedTag];
    //NSLog(@"selected: %d", selected);
    [windowPopup removeAllItems];
    
    while (win = (NSWindow *)[winEnum nextObject])
    {
        int winNum = [win windowNumber];
        NSString *winTitle = [win title];
        NSString *winStr = [NSString stringWithFormat: @"%@ %d", winTitle, winNum];
        [windowPopup addItemWithTitle: winStr];
        NSMenuItem *item = [windowPopup itemWithTitle: winStr];
        [item setTag: winNum];
    }
    
    [windowPopup selectItemWithTag: selected];
}

- (void)showViewsForCurrWindow
{
    [outlineView reloadData];
    [outlineView expandItem: [self currContentView]
             expandChildren: NO];
}

#pragma mark actions

- (IBAction)windowListPopupAction:(id)sender
{
    int winNum = [windowPopup selectedTag];
    NSWindow *win = [self windowWithWindowNum: winNum];
    if (win == nil)
    {
        NSLog(@"Window with windowNum %d not found", winNum);
        return;
    }
    
    [win orderFront: nil];
    [self setCurrWindow: win];
    [self showViewsForCurrWindow];
}

- (IBAction)refreshBtnAction:(id)sender
{
    if ([self currWindow] != nil)
    {
        [self showViewsForCurrWindow];
    }
}

@end
